# Generated by Django 2.2.8 on 2020-02-10 09:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("caluma_workflow", "0016_auto_20200113_0727")]

    operations = [
        migrations.AlterField(
            model_name="case",
            name="created_at",
            field=models.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="case",
            name="created_by_user",
            field=models.CharField(
                blank=True, db_index=True, max_length=150, null=True
            ),
        ),
        migrations.AlterField(
            model_name="case",
            name="modified_at",
            field=models.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="flow",
            name="created_at",
            field=models.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="flow",
            name="created_by_user",
            field=models.CharField(
                blank=True, db_index=True, max_length=150, null=True
            ),
        ),
        migrations.AlterField(
            model_name="flow",
            name="modified_at",
            field=models.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalcase",
            name="created_at",
            field=models.DateTimeField(blank=True, db_index=True, editable=False),
        ),
        migrations.AlterField(
            model_name="historicalcase",
            name="created_by_user",
            field=models.CharField(
                blank=True, db_index=True, max_length=150, null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalcase",
            name="modified_at",
            field=models.DateTimeField(blank=True, db_index=True, editable=False),
        ),
        migrations.AlterField(
            model_name="historicalflow",
            name="created_at",
            field=models.DateTimeField(blank=True, db_index=True, editable=False),
        ),
        migrations.AlterField(
            model_name="historicalflow",
            name="created_by_user",
            field=models.CharField(
                blank=True, db_index=True, max_length=150, null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalflow",
            name="modified_at",
            field=models.DateTimeField(blank=True, db_index=True, editable=False),
        ),
        migrations.AlterField(
            model_name="historicaltask",
            name="created_at",
            field=models.DateTimeField(blank=True, db_index=True, editable=False),
        ),
        migrations.AlterField(
            model_name="historicaltask",
            name="created_by_user",
            field=models.CharField(
                blank=True, db_index=True, max_length=150, null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicaltask",
            name="modified_at",
            field=models.DateTimeField(blank=True, db_index=True, editable=False),
        ),
        migrations.AlterField(
            model_name="historicaltaskflow",
            name="created_at",
            field=models.DateTimeField(blank=True, db_index=True, editable=False),
        ),
        migrations.AlterField(
            model_name="historicaltaskflow",
            name="created_by_user",
            field=models.CharField(
                blank=True, db_index=True, max_length=150, null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicaltaskflow",
            name="modified_at",
            field=models.DateTimeField(blank=True, db_index=True, editable=False),
        ),
        migrations.AlterField(
            model_name="historicalworkflow",
            name="created_at",
            field=models.DateTimeField(blank=True, db_index=True, editable=False),
        ),
        migrations.AlterField(
            model_name="historicalworkflow",
            name="created_by_user",
            field=models.CharField(
                blank=True, db_index=True, max_length=150, null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalworkflow",
            name="modified_at",
            field=models.DateTimeField(blank=True, db_index=True, editable=False),
        ),
        migrations.AlterField(
            model_name="historicalworkitem",
            name="created_at",
            field=models.DateTimeField(blank=True, db_index=True, editable=False),
        ),
        migrations.AlterField(
            model_name="historicalworkitem",
            name="created_by_user",
            field=models.CharField(
                blank=True, db_index=True, max_length=150, null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalworkitem",
            name="modified_at",
            field=models.DateTimeField(blank=True, db_index=True, editable=False),
        ),
        migrations.AlterField(
            model_name="task",
            name="created_at",
            field=models.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="task",
            name="created_by_user",
            field=models.CharField(
                blank=True, db_index=True, max_length=150, null=True
            ),
        ),
        migrations.AlterField(
            model_name="task",
            name="modified_at",
            field=models.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="taskflow",
            name="created_at",
            field=models.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="taskflow",
            name="created_by_user",
            field=models.CharField(
                blank=True, db_index=True, max_length=150, null=True
            ),
        ),
        migrations.AlterField(
            model_name="taskflow",
            name="modified_at",
            field=models.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="workflow",
            name="created_at",
            field=models.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="workflow",
            name="created_by_user",
            field=models.CharField(
                blank=True, db_index=True, max_length=150, null=True
            ),
        ),
        migrations.AlterField(
            model_name="workflow",
            name="modified_at",
            field=models.DateTimeField(auto_now=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="workitem",
            name="created_at",
            field=models.DateTimeField(auto_now_add=True, db_index=True),
        ),
        migrations.AlterField(
            model_name="workitem",
            name="created_by_user",
            field=models.CharField(
                blank=True, db_index=True, max_length=150, null=True
            ),
        ),
        migrations.AlterField(
            model_name="workitem",
            name="modified_at",
            field=models.DateTimeField(auto_now=True, db_index=True),
        ),
    ]
