# Generated by Django 2.2.24 on 2022-01-10 10:51

from django.db import migrations
import localized_fields.fields.field


class Migration(migrations.Migration):

    dependencies = [
        ("caluma_form", "0041_action_button_question"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalquestion",
            name="hint_text",
            field=localized_fields.fields.field.LocalizedField(
                blank=True, null=True, required=[]
            ),
        ),
        migrations.AddField(
            model_name="question",
            name="hint_text",
            field=localized_fields.fields.field.LocalizedField(
                blank=True, null=True, required=[]
            ),
        ),
    ]
