# Generated by Django 2.2.10 on 2020-02-26 09:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("caluma_form", "0032_auto_20200220_1311")]

    operations = [
        migrations.AlterField(
            model_name="form",
            name="slug",
            field=models.SlugField(max_length=127, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name="historicalform",
            name="slug",
            field=models.SlugField(max_length=127),
        ),
        migrations.AlterField(
            model_name="historicaloption",
            name="slug",
            field=models.SlugField(max_length=127),
        ),
        migrations.AlterField(
            model_name="historicalquestion",
            name="slug",
            field=models.SlugField(max_length=127),
        ),
        migrations.AlterField(
            model_name="option",
            name="slug",
            field=models.SlugField(max_length=127, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name="question",
            name="slug",
            field=models.SlugField(max_length=127, primary_key=True, serialize=False),
        ),
    ]
