# Generated by Django 3.2.12 on 2022-03-09 11:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("caluma_workflow", "0028_redoable_field"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicaltask",
            name="continue_async",
            field=models.BooleanField(
                default=False,
                help_text="Whether to continue the flow if the multiple instance work item has ready siblings",
            ),
        ),
        migrations.AddField(
            model_name="task",
            name="continue_async",
            field=models.BooleanField(
                default=False,
                help_text="Whether to continue the flow if the multiple instance work item has ready siblings",
            ),
        ),
    ]
