# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-02-06 08:54
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("caluma_form", "0003_auto_20190130_0920"),
        ("caluma_workflow", "0006_auto_20190121_1440"),
    ]

    operations = [
        migrations.RemoveField(model_name="workflow", name="form"),
        migrations.AddField(
            model_name="workflow",
            name="allow_all_forms",
            field=models.BooleanField(
                default=False, help_text="Allow workflow to be started with any form"
            ),
        ),
        migrations.AddField(
            model_name="workflow",
            name="allow_forms",
            field=models.ManyToManyField(
                blank=True,
                help_text="List of forms which are allowed to start workflow with",
                related_name="workflows",
                to="caluma_form.Form",
            ),
        ),
    ]
